// ServletDirectoryFrame.java
// $Id: ServletDirectoryFrame.java,v 1.5 1998/05/25 15:52:22 bmahe Exp $
// (c) COPYRIGHT MIT and INRIA, 1996.
// Please first read the full copyright statement in file COPYRIGHT.html

package org.w3c.jigsaw.servlet;

import java.io.*;
import java.net.*;
import java.util.*;

import javax.servlet.*;

import org.w3c.tools.resources.store.*;
import org.w3c.tools.resources.*;
import org.w3c.jigsaw.frames.*;

/**
 *  @author Alexandre Rafalovitch <alex@access.com.au>
 *  @author Anselm Baird-Smith <abaird@w3.org>
 *  @author Benoit Mahe <bmahe@w3.org>
 */

public class ServletDirectoryFrame extends HTTPFrame {

  public File getDirectory() {
    if (dresource != null)
      return dresource.getDirectory();
    return null;
  }

  /**
   * ServletContext implementation - Lookup a given servlet.
   */

  public Servlet getServlet(String name) {
    if (dresource != null) {
      ResourceReference rr = dresource.lookup(name);
      try {
	Resource resource = rr.lock();
	if (resource instanceof ServletWrapper)
	  return ((ServletWrapper) resource).getServlet();
      } catch (InvalidResourceException ex) {
	return null;
      } finally {
	rr.unlock();
      }
    }
    return null;
  }

  /**
   * ServletContext implementation - Enumerate all servlets within context.
   */

  public Enumeration getServlets() {
    if (dresource != null)
      return new ServletEnumeration(this, 
				    dresource.enumerateResourceIdentifiers());
    else
      return new ServletEnumeration(this, null);
  }

    /**
     * ServletContext implementation - Enumerate all servlets names
     * within context.
     */

    public Enumeration getServletNames() {
	if (dresource != null)
	    return new ServletNamesEnumeration(this, 
			       dresource.enumerateResourceIdentifiers());
	else
	    return new ServletNamesEnumeration(this, null);
    }

  /**
   * ServletContext implementation - Get server informations.
   */

  public String getServerInfo() {
    return resource.getServer().getSoftware();
  }

  /**
   * ServletContext implementation - Get an attribute value.
   * We map this into the ServletWrapper attributes, without
   * support for name clashes though.
   * @param name The attribute name.
   */

  public Object getAttribute(String name) {
    if ( definesAttribute(name) )
      return getValue(name, null);
    else if (resource.definesAttribute(name))
      return resource.getValue(name, null);
    return null;
  }

  /**
   * We add a <em>context</em> attribute to all our children.
   * The <em>context</em> attribute is any object implementing the
   * ServletContext interface.
   */

  protected void updateDefaultChildAttributes(Hashtable attrs) {
      attrs.put("servlet-context", 
		new JigsawServletContext(getFrameReference()));
  }

}
